/*!
 *****************************************************************************
 * @file:   ADuCM410_Setup.c
 * @brief:  setup for ADuCM410
 *-----------------------------------------------------------------------------
 *
Copyright (c) 2013-2018 Analog Devices, Inc. All Rights Reserved.

This software is proprietary to Analog Devices, Inc. and its licensors.
By using this software you agree to the terms of the associated
Analog Devices Software License Agreement.
 ******************************************************************************/

#include "GptLib.h"
#include "ClkLib.h"
#include "DacLib.h"
#include "DmaLib.h"
#include "AdcLib.h"

/*
   User can change following cofiguraion according to application
*/
CLK_SETUP_t gClkSetup = 
{
   //--------- configuration ---------
   .pllEn = 1,
   .rootClkMux = ENUM_CLOCK_CLKCON0_CLKMUX_SPLL,
   .analogClkMux = ENUM_CLOCK_CLKCON0_ANACLKMUX_HFOSC,
    

//-----------------Clock Divider -------------   
   .hclkDiv = ENUM_CLOCK_CLKCON1_CDHCLK_DIV1,
   .pclk0Div = ENUM_CLOCK_CLKCON1_CDPCLK0_DIV8,
   .pclk1Div = ENUM_CLOCK_CLKCON1_CDPCLK1_DIV8,
    
   .clockOff = 0x00000000,  //all clock on

   //---------Interrupt------------
   .pllIntEn = 0,    //0 - disalbe interrrupt related to PLL

   //-------TEST CLOCK -------
   .clkOutput = ENUM_CLOCK_CLKCON0_CLKOUT_B1_PCLK,
};

DMA_SETUP_t gDmaSetup =
{             
   .chanEn = TIMERPLA0_B|ADC_B,   //enable timer trigger and ADC
   .chanMask = 0, //no mask
   .useAlt = 0,   //use primary structure
   .priority = 0, //use defualt priority based on channel number
   .byteSwap = 0, //no byte swap
   .srcAddrDec = 0,  //no decreament
   .dstAddrDec = 0,  //no decreament
                //pointer to DMA channel description
   .pDmaChxDesc = NULL, //if NULL, it will be initialized in setup function
};

GPT_SETUP_t gGpt0Setup = 
{
   .clockSource = ENUM_TMR_CON_CLK_HCLK,
   .prescaler=ENUM_TMR_CON_PRE_DIV16,
   .countUp = 0,  //counting down
   .periodicMode = 1,
   .loadValue = 1250,   //160M/16/1250 = 8000Hz
   .reload = 1,
};

GPT_SETUP_t gGpt2Setup = 
{
   .clockSource = ENUM_TMR_CON_CLK_HCLK,
   .prescaler=ENUM_TMR_CON_PRE_DIV16,
   .countUp = 0,  //counting down
   .periodicMode = 1,
   .loadValue = 1250,
   .reload = 1,
};

ADC_SETUP_t gAdcSetup = 
{
   .adcPwrDown = 0,  //power up ADC
   .dmaEnForNonSeq = 1, //DMA for non-sequence mode
   .dmaEnForSeq = 0, //no DMA for sequnce mode  
   .pinMode = ENUM_ADC_ADCCON_PINMOD_PIN_EDGE,  
   .timerTrigConv = 1,  //ADC conversion triggered by timer event 
   .osr = ENUM_ADC_ADCCON_OSR_OSR_NS1, //no oversample
   .cnvFreqHighDiv = 160,   //conversion frequency = ADC clock(16M)/16 = 1MHz
   .cnvFreqLowDiv = 1600,     //conversion frequency = ADC clock(16M/160 = 100KHz
   .bypassPChanPreBuf = 0, //P channel precharge buffer
   .bypassNChanPreBuf = 0, //N channel precharge buffer
//------------ sequnce ---------
   .seqInterval = 255,  //run sequence once
   .seqEn = 0,       //enable sequence
//---------- Interrupt setup -------
   .CnvIntEn = 0, //disable ADC conversion interrupt
   .seqIntEn = 0, //disable sequence interrupt
   .refBufPwrDown = 0,  //reference buffer power up
};




DAC_SETUP_t gDACSetup = 
{
    // DAC0
    .dacChan[0] = 
    {
        .driveAblityEn = ENUM_DAC_DACCON_N__DRV_NORMAL, // drive ability normal
        .powerDown = 0, // DAC Top enable
        .fullScale = ENUM_DAC_DACCON_N__FSLVL_OUT2P5, // DAC fullscale is 2.5V
        .dataClear = ENUM_DAC_DACCON_N__EN_CLEAROFF, // DAC data normal input
        // data[27:16] -- 0x800(12-bit DAC data), data[15:12] -- 0x00(4-bit DAC data), 
        .data = 0x098AD000,
    },

    // DAC1
    .dacChan[1] =
    {
        .driveAblityEn = ENUM_DAC_DACCON_N__DRV_NORMAL, // drive ability normal
        .powerDown = 0, // DAC Top enable
        .fullScale = ENUM_DAC_DACCON_N__FSLVL_OUT2P5, // DAC fullscale is 2.5V
        .dataClear = ENUM_DAC_DACCON_N__EN_CLEAROFF, // DAC data normal input
        // data[27:16] -- 0x800(12-bit DAC data), data[15:12] -- 0x00(4-bit DAC data), 
        .data = 0x0d8AD000, 
    },

    // DAC2
    .dacChan[2] =
    {
        .driveAblityEn = ENUM_DAC_DACCON_N__DRV_NORMAL, // drive ability normal
        .powerDown = 0, // DAC Top enable
        .fullScale = ENUM_DAC_DACCON_N__FSLVL_OUT2P5, // DAC fullscale is 2.5V
        .dataClear = ENUM_DAC_DACCON_N__EN_CLEAROFF, // DAC data normal input
        // data[27:16] -- 0x800(12-bit DAC data), data[15:12] -- 0x00(4-bit DAC data), 
        .data = 0, 
    },

    // DAC3
    .dacChan[3] =
    {
        .driveAblityEn = ENUM_DAC_DACCON_N__DRV_NORMAL, // drive ability normal
        .powerDown = 0, // DAC Top enable
        .fullScale = ENUM_DAC_DACCON_N__FSLVL_OUT2P5, // DAC fullscale is 2.5V
        .dataClear = ENUM_DAC_DACCON_N__EN_CLEAROFF, // DAC data normal input
        // data[27:16] -- 0x800(12-bit DAC data), data[15:12] -- 0x00(4-bit DAC data), 
        .data = 0x08000000, 
    },
    
    // DAC4
    .dacChan[4] = 
    {
        .driveAblityEn = ENUM_DAC_DACCON_N__DRV_NORMAL, // drive ability normal
        .powerDown = 0, // DAC Top enable
        .fullScale = ENUM_DAC_DACCON_N__FSLVL_OUT2P5, // DAC fullscale is 2.5V
        .dataClear = ENUM_DAC_DACCON_N__EN_CLEAROFF, // DAC data normal input
        // data[27:16] -- 0x800(12-bit DAC data), data[15:12] -- 0x00(4-bit DAC data), 
        .data = 0x08000000, 
    },

    // DAC5
    .dacChan[5] = 
    {
        .driveAblityEn = ENUM_DAC_DACCON_N__DRV_NORMAL, // drive ability normal
        .powerDown = 0, // DAC Top enable
        .fullScale = ENUM_DAC_DACCON_N__FSLVL_OUT2P5, // DAC fullscale is 2.5V
        .dataClear = ENUM_DAC_DACCON_N__EN_CLEAROFF, // DAC data normal input
        // data[27:16] -- 0x800(12-bit DAC data), data[15:12] -- 0x00(4-bit DAC data), 
        .data = 0x08000000, 
    },

    // DAC6
    .dacChan[6] = 
    {
        .driveAblityEn = ENUM_DAC_DACCON_N__DRV_NORMAL, // drive ability normal
        .powerDown = 0, // DAC Top enable
        .fullScale = ENUM_DAC_DACCON_N__FSLVL_OUT2P5, // DAC fullscale is 2.5V
        .dataClear = ENUM_DAC_DACCON_N__EN_CLEAROFF, // DAC data normal input
        // data[27:16] -- 0x800(12-bit DAC data), data[15:12] -- 0x00(4-bit DAC data), 
        .data = 0x08000000, 
    },

    // DAC7
    .dacChan[7] = 
    {
        .driveAblityEn = ENUM_DAC_DACCON_N__DRV_NORMAL, // drive ability normal
        .powerDown = 0, // DAC Top enable
        .fullScale = ENUM_DAC_DACCON_N__FSLVL_OUT2P5, // DAC fullscale is 2.5V
        .dataClear = ENUM_DAC_DACCON_N__EN_CLEAROFF, // DAC data normal input
        // data[27:16] -- 0x800(12-bit DAC data), data[15:12] -- 0x00(4-bit DAC data), 
        .data = 0x08000000, 
    },

    // DAC8
    .dacChan[8] = 
    {
        .driveAblityEn = ENUM_DAC_DACCON_N__DRV_NORMAL, // drive ability normal
        .powerDown = 1,
        .fullScale = ENUM_DAC_DACCON_N__FSLVL_OUT2P5, // DAC fullscale is 2.5V
        .dataClear = ENUM_DAC_DACCON_N__EN_CLEAROFF, // DAC data normal input
        // data[27:16] -- 0x800(12-bit DAC data), data[15:12] -- 0x00(4-bit DAC data), 
        .data = 0x08000000, 
    },

    // DAC9
    .dacChan[9] = 
    {
        .driveAblityEn = ENUM_DAC_DACCON_N__DRV_NORMAL, // drive ability normal
        .powerDown = 1, 
        .fullScale = ENUM_DAC_DACCON_N__FSLVL_OUT2P5, // DAC fullscale is 2.5V
        .dataClear = ENUM_DAC_DACCON_N__EN_CLEAROFF, // DAC data normal input
        // data[27:16] -- 0x800(12-bit DAC data), data[15:12] -- 0x00(4-bit DAC data), 
        .data = 0x08000000, 
    },

    // DAC10
    .dacChan[10] = 
    {
        .driveAblityEn = ENUM_DAC_DACCON_N__DRV_NORMAL, // drive ability normal
        .powerDown = 1, 
        .fullScale = ENUM_DAC_DACCON_N__FSLVL_OUT2P5, // DAC fullscale is 2.5V
        .dataClear = ENUM_DAC_DACCON_N__EN_CLEAROFF, // DAC data normal input
        // data[27:16] -- 0x800(12-bit DAC data), data[15:12] -- 0x00(4-bit DAC data), 
        .data = 0x08000000, 
    },

    // DAC11
    .dacChan[11] = 
    {
        .driveAblityEn = ENUM_DAC_DACCON_N__DRV_NORMAL, // drive ability normal
        .powerDown = 1, 
        .fullScale = ENUM_DAC_DACCON_N__FSLVL_OUT2P5, // DAC fullscale is 2.5V
        .dataClear = ENUM_DAC_DACCON_N__EN_CLEAROFF, // DAC data normal input
        // data[27:16] -- 0x800(12-bit DAC data), data[15:12] -- 0x00(4-bit DAC data), 
        .data = 0x08000000, 
    },
};



