/*!
 *****************************************************************************
 * @file:
 * @brief:
 *-----------------------------------------------------------------------------
 *
Copyright (c) 2013-2018 Analog Devices, Inc. All Rights Reserved.

This software is proprietary to Analog Devices, Inc. and its licensors.
By using this software you agree to the terms of the associated
Analog Devices Software License Agreement.
 ******************************************************************************/

#include "ADuCM410.h"
#include "WdtLib.h"
#include "common.h"
#include "DioLib.h"
#include "AdcLib.h"
#include "ClkLib.h"
#include "GptLib.h"
#include "DacLib.h"
#include "DmaLib.h"
#include "AdcLib.h"
#include "WaveDat.h"

#ifdef REDIRECT_OUTPUT_TO_UART
   #define ADC_DATA_LENGTH 400
#else
   #define ADC_DATA_LENGTH 4
#endif

//Varaibles decalaration
extern volatile uint32_t DmaDoneCnt;
extern volatile uint32_t conversionDone;
extern volatile uint32_t dataCnt;
extern volatile uint32_t data;
extern volatile uint32_t sta;
extern uint32_t AdcData[ADC_DATA_LENGTH];
extern volatile uint32_t gAdcDmaDoneCnt;
extern volatile int32_t gAdcNumLeft;

//Function decalaration
void delay(uint32_t t);
void DmaInit(void);
void AdcInit(void);
void DacInit(void);
void PlaForAdcInit(void);


uint32_t AdcData[ADC_DATA_LENGTH] = {0};
volatile uint32_t gAdcDmaDoneCnt = 0;
volatile int32_t gAdcNumLeft = ADC_DATA_LENGTH;
volatile uint32_t DmaDoneCnt = 0;
volatile uint32_t conversionDone = 0;
volatile uint32_t dataCnt = 0;
volatile uint32_t data = 0;
volatile uint32_t sta = 0;


void DacInit()
{
   //DAC setup
    DioCfgPin(pADI_GPIO4,PIN0,1);
	 DioCfgPin(pADI_GPIO4,PIN1,1);
	 DioCfgPin(pADI_GPIO4,PIN2,1);
	 DioCfgPin(pADI_GPIO4,PIN4,1);
	 DioCfgPin(pADI_GPIO5,PIN0,2);
	 DioCfgPin(pADI_GPIO5,PIN1,2);
	 DioCfgPin(pADI_GPIO5,PIN2,2);
	 DioCfgPin(pADI_GPIO5,PIN3,2);
    DACSetup(&gDACSetup); 
}

int main (void)
{
    DmaDoneCnt = 0;
    conversionDone = 0;
    dataCnt = 0;
    data = 0;
    sta = 0;
    gAdcDmaDoneCnt = 0;
    gAdcNumLeft = ADC_DATA_LENGTH;
    WdtGo(false);

    ClkSetup(&gClkSetup);

    SystemInit();
    INIT();
    SafetyWait();
    
    DacInit();
    DmaInit();
    GptSetup(pADI_GPT0,&gGpt0Setup);
    NVIC_EnableIRQ(GPT0_IRQn);
    
    AdcInit();
    GptSetup(pADI_GPT2,&gGpt2Setup);
   
    GptGo(pADI_GPT0);   //start timer, DMA request triggerd when timeout
    while(pADI_GPT0->VAL>=625);  //half update rate delay
    GptGo(pADI_GPT2);   //start timer, DMA request triggerd when timeout
    
    while(conversionDone==0);   //wait for conversion finish
	  
    DEBUG_MESSAGE("ADC raw data, voltage(mV)");
    for(uint32_t i=0;i<ADC_DATA_LENGTH;i++)
    {
		if(AdcData[i]&BITM_ADC_ADCDAT_N__RDY)
		{
		    AdcData[i] = AdcData[i]>>BITP_ADC_ADCDAT_N__DAT;
			 //DEBUG_MESSAGE("%d",AdcData[i]);
          DEBUG_MESSAGE("%d,%f",AdcData[i],AdcData[i]/65536.0*2500);

		}
	 	else
		{
			DEBUG_MESSAGE("ERROR: interrupt without data ready, code:0x%x",AdcData[i]);
		}

    }


    while(1)
    {
        delay(10000);
        //WdtRefresh();
    }

}

void delay(uint32_t t)
{
   uint32_t i;
   for(i=0;i<t;i++);
}

void DmaInit()
{
   DmaSetup(&gDmaSetup);

   gDmaSetup.pDmaChxDesc[TIMERPLA0_C].ctrlCfg.Bits.dst_inc = ENUM_DMA_INC_NO;   
   gDmaSetup.pDmaChxDesc[TIMERPLA0_C].ctrlCfg.Bits.src_inc = ENUM_DMA_INC_WORD;   
   gDmaSetup.pDmaChxDesc[TIMERPLA0_C].ctrlCfg.Bits.src_size = SIZE_WORD; 
   gDmaSetup.pDmaChxDesc[TIMERPLA0_C].srcEndPtr = (uint32_t)((uint32_t)gWaveDat + (WAVE_DATA_BUFFER_LEN/2- 1)*4);
   gDmaSetup.pDmaChxDesc[TIMERPLA0_C].destEndPtr = (uint32_t)(&pADI_VDAC->DACDAT1);
   gDmaSetup.pDmaChxDesc[TIMERPLA0_C].ctrlCfg.Bits.r_power = 0;   //2^0, 1 transaction each request
   gDmaSetup.pDmaChxDesc[TIMERPLA0_C].ctrlCfg.Bits.n_minus_1 = WAVE_DATA_BUFFER_LEN/2-1;
   gDmaSetup.pDmaChxDesc[TIMERPLA0_C].ctrlCfg.Bits.cycle_ctrl = DMA_PING;
   
   gDmaSetup.pDmaChxDesc[ALT_TIMERPLA0_C].ctrlCfg.Bits.dst_inc = ENUM_DMA_INC_NO;   
   gDmaSetup.pDmaChxDesc[ALT_TIMERPLA0_C].ctrlCfg.Bits.src_inc = ENUM_DMA_INC_WORD;   
   gDmaSetup.pDmaChxDesc[ALT_TIMERPLA0_C].ctrlCfg.Bits.src_size = SIZE_WORD; 
   gDmaSetup.pDmaChxDesc[ALT_TIMERPLA0_C].srcEndPtr = (uint32_t)((uint32_t)gWaveDat + (WAVE_DATA_BUFFER_LEN- 1)*4);
   gDmaSetup.pDmaChxDesc[ALT_TIMERPLA0_C].destEndPtr = (uint32_t)(&pADI_VDAC->DACDAT1);
   gDmaSetup.pDmaChxDesc[ALT_TIMERPLA0_C].ctrlCfg.Bits.r_power = 0;   //2^0, 1 transaction each request
   gDmaSetup.pDmaChxDesc[ALT_TIMERPLA0_C].ctrlCfg.Bits.n_minus_1 = WAVE_DATA_BUFFER_LEN/2-1;
   gDmaSetup.pDmaChxDesc[ALT_TIMERPLA0_C].ctrlCfg.Bits.cycle_ctrl = DMA_PING;
//
   NVIC_EnableIRQ(DMA_TRIG0_IRQn);
   
   DioCfgPin(pADI_GPIO0,PIN5,P0_5_PLA3_OUT);
   pADI_DMAREQ->REQEN |=BITM_DMAREQ_REQEN_GPLA_DMA_EN0;//channel 0 eanbled

   //element 3 setup
    pADI_PLA->PLA_ELEM3 = (ENUM_PLA_PLA_ELEM_N__MUX1_ELEM1<<BITP_PLA_PLA_ELEM_N__MUX1) \
                            |(ENUM_PLA_PLA_ELEM_N__MUX3_ODD<<BITP_PLA_PLA_ELEM_N__MUX3) \
                            |(ENUM_PLA_PLA_ELEM_N__MUX4_FF<<BITP_PLA_PLA_ELEM_N__MUX4)  \
                            |(ENUM_PLA_PLA_ELEM_N__TBL_NOTB<<BITP_PLA_PLA_ELEM_N__TBL);
    pADI_PLA->PLA_CLK |= ENUM_PLA_PLA_CLK_BLOCK0_T0<<BITP_PLA_PLA_CLK_BLOCK0;
    //element 1 setup, assign element 1 output = element 3 output
    pADI_PLA->PLA_ELEM1 = (ENUM_PLA_PLA_ELEM_N__MUX1_ELEM3<<BITP_PLA_PLA_ELEM_N__MUX1) \
                        |(ENUM_PLA_PLA_ELEM_N__MUX3_ODD<<BITP_PLA_PLA_ELEM_N__MUX3) \
                        |(ENUM_PLA_PLA_ELEM_N__MUX4_BYPASS<<BITP_PLA_PLA_ELEM_N__MUX4)  \
                        |(ENUM_PLA_PLA_ELEM_N__TBL_B<<BITP_PLA_PLA_ELEM_N__TBL);
    pADI_PLA->PLA_IRQ0 = (BITM_PLA_PLA_IRQ0_IRQ0_EN|3);   //request from PLA3
    
    pADI_DMAREQ->REQ0SEL = ENUM_DMAREQ_REQ0SEL_DMA_REQ0_SEL_PLA_REQ0;//select PLA DMA request 0
    pADI_DMAREQ->PLAREQEN |= BITM_DMAREQ_PLAREQEN_PLA_DMA_REQ0_EN;

   
}

void PlaForAdcInit()
{
    //element 29 setup
    pADI_PLA->PLA_ELEM29 = (ENUM_PLA_PLA_ELEM_N__MUX1_ELEM1<<BITP_PLA_PLA_ELEM_N__MUX1) \
                            |(ENUM_PLA_PLA_ELEM_N__MUX3_ODD<<BITP_PLA_PLA_ELEM_N__MUX3) \
                            |(ENUM_PLA_PLA_ELEM_N__MUX4_FF<<BITP_PLA_PLA_ELEM_N__MUX4)  \
                            |(ENUM_PLA_PLA_ELEM_N__TBL_NOTB<<BITP_PLA_PLA_ELEM_N__TBL);
    pADI_PLA->PLA_CLK |= ENUM_PLA_PLA_CLK_BLOCK1_T2<<BITP_PLA_PLA_CLK_BLOCK1;
    //ADC conversion source from element29 output
    pADI_PLA->PLA_ADC = BITM_PLA_PLA_ADC_CONVST_EN|(29<<BITP_PLA_PLA_ADC_CONVST_SRC);

    //element 17 setup, assign element 17 output = element 29 output
    pADI_PLA->PLA_ELEM17 = (ENUM_PLA_PLA_ELEM_N__MUX1_ELEM13<<BITP_PLA_PLA_ELEM_N__MUX1) \
                        |(ENUM_PLA_PLA_ELEM_N__MUX3_ODD<<BITP_PLA_PLA_ELEM_N__MUX3) \
                        |(ENUM_PLA_PLA_ELEM_N__MUX4_BYPASS<<BITP_PLA_PLA_ELEM_N__MUX4)  \
                        |(ENUM_PLA_PLA_ELEM_N__TBL_B<<BITP_PLA_PLA_ELEM_N__TBL);
}
void AdcInit()
{
    conversionDone = 0;
    dataCnt = 0;
    data = 0;
    sta = 0;

    AdcSetup(&gAdcSetup);
    AdcPinSingleExt(SING_END_CHAN_AIN1_REFN);
    
    PlaForAdcInit();
    NVIC_EnableIRQ(DMA_ADC_IRQn);//enable interrupt

    gDmaSetup.pDmaChxDesc[ADC_C].ctrlCfg.Bits.dst_inc = ENUM_DMA_INC_WORD;   
    gDmaSetup.pDmaChxDesc[ADC_C].ctrlCfg.Bits.src_inc = ENUM_DMA_INC_NO;   
    gDmaSetup.pDmaChxDesc[ADC_C].ctrlCfg.Bits.src_size = SIZE_WORD; 
    gDmaSetup.pDmaChxDesc[ADC_C].srcEndPtr = (uint32_t)(&REG_ADC_REF_AIN1_DAT1);
    gDmaSetup.pDmaChxDesc[ADC_C].ctrlCfg.Bits.r_power = 0;   //2^0, 1 transaction each request
    
#if   ADC_DATA_LENGTH <= 1024
      gDmaSetup.pDmaChxDesc[ADC_C].destEndPtr = (uint32_t)((uint32_t)AdcData + (ADC_DATA_LENGTH -1)*4);
      gDmaSetup.pDmaChxDesc[ADC_C].ctrlCfg.Bits.n_minus_1 = ADC_DATA_LENGTH -1;
      gDmaSetup.pDmaChxDesc[ADC_C].ctrlCfg.Bits.cycle_ctrl = DMA_BASIC;   
#else  
      uint32_t remainNum = 0;
      remainNum = ADC_DATA_LENGTH - 1024;
      gDmaSetup.pDmaChxDesc[ADC_C].destEndPtr = (uint32_t)((uint32_t)AdcData + 1023*4);
      gDmaSetup.pDmaChxDesc[ADC_C].ctrlCfg.Bits.n_minus_1 = 1023;
      gDmaSetup.pDmaChxDesc[ADC_C].ctrlCfg.Bits.cycle_ctrl = DMA_PING;
      
      gDmaSetup.pDmaChxDesc[ALT_ADC_C].ctrlCfg.Bits.dst_inc = ENUM_DMA_INC_WORD;   
      gDmaSetup.pDmaChxDesc[ALT_ADC_C].ctrlCfg.Bits.src_inc = ENUM_DMA_INC_NO;   
      gDmaSetup.pDmaChxDesc[ALT_ADC_C].ctrlCfg.Bits.src_size = SIZE_WORD; 
      gDmaSetup.pDmaChxDesc[ALT_ADC_C].srcEndPtr = (uint32_t)(&REG_ADC_REF_AIN1_DAT1);
      gDmaSetup.pDmaChxDesc[ALT_ADC_C].ctrlCfg.Bits.r_power = 0;   //2^0, 1 transaction each request
      
      if(remainNum<=1024)
      {
         gDmaSetup.pDmaChxDesc[ALT_ADC_C].destEndPtr = (uint32_t)((uint32_t)AdcData + 1024*4 + (remainNum -1)*4);
         gDmaSetup.pDmaChxDesc[ALT_ADC_C].ctrlCfg.Bits.n_minus_1 = remainNum-1;
      }
      else
      {
         gDmaSetup.pDmaChxDesc[ALT_ADC_C].destEndPtr = (uint32_t)((uint32_t)AdcData + (2*1024 -1)*4);
         gDmaSetup.pDmaChxDesc[ALT_ADC_C].ctrlCfg.Bits.n_minus_1 = 1024-1; 
      }
      gDmaSetup.pDmaChxDesc[ALT_ADC_C].ctrlCfg.Bits.cycle_ctrl = DMA_PING;
#endif
     
    AdcGo(ENUM_ADC_ADCCON_CONVTYPE_PLA);//ADC conversion triggered by Timer
}

void DMA_TRIG0_Int_Handler(void)
{
   
   DmaDoneCnt++;

   DmaDesc *Desc;
   if((DmaDoneCnt%2) ==  0) //config alternate structure
   {
      Desc = &(gDmaSetup.pDmaChxDesc[ALT_TIMERPLA0_C]);
      Desc->srcEndPtr = (uint32_t)((uint32_t)gWaveDat + (WAVE_DATA_BUFFER_LEN- 1)*4);
   }
   else
   {
      Desc = &(gDmaSetup.pDmaChxDesc[TIMERPLA0_C]);
      Desc->srcEndPtr = (uint32_t)((uint32_t)gWaveDat + (WAVE_DATA_BUFFER_LEN/2- 1)*4);
   }
   
   //Desc->srcEndPtr = (uint32_t)((uint32_t)gWaveDat + (WAVE_DATA_BUFFER_LEN- 1)*4);
   Desc->ctrlCfg.Bits.n_minus_1 = WAVE_DATA_BUFFER_LEN/2 -1;
   
   Desc->ctrlCfg.Bits.cycle_ctrl = DMA_PING;
   
}

//interrupt handler
void DMA_ADC_Int_Handler()
{
   int32_t numNext;
   DmaDesc *Desc;
   gAdcDmaDoneCnt++;
   gAdcNumLeft -= 1024;
   if(gAdcNumLeft <= 0)  //less than 1024 samples
   {
      //stop DMA, stop conversion
      AdcGo(ENUM_ADC_ADCCON_CONVTYPE_IDLE);
      conversionDone = 1;
   }
   else  /*Ping-Pong mode*/
   {
      if(gAdcNumLeft>1024)
      {
         numNext = gAdcNumLeft - 1024;
         if((gAdcDmaDoneCnt%2) ==  0) //config alternate structure
            Desc = &(gDmaSetup.pDmaChxDesc[ALT_ADC_C]);
         else  //config primary structure
            Desc = &(gDmaSetup.pDmaChxDesc[ADC_C]);

         if(numNext > 1024)
         {
            Desc->destEndPtr = ((uint32_t)AdcData)+ ((gAdcDmaDoneCnt+1)<<12)+((1024-1)<<2);
            Desc->ctrlCfg.Bits.n_minus_1 = 1023;
         }
         else
         {
            Desc->destEndPtr = ((uint32_t)AdcData)+ ((gAdcDmaDoneCnt+1)<<12)+((uint32_t)(numNext-1)<<2);
            Desc->ctrlCfg.Bits.n_minus_1 = (uint32_t)(numNext -1);
         }
         Desc->ctrlCfg.Bits.cycle_ctrl = DMA_PING;
      }
   }
}

void GP_Tmr0_Int_Handler()
{
   DEBUG_PIN1_TOGGLE();
}








